{% extends "admin/base_site.html" %}
{% load i18n %}
{% load url from future %}

{% block coltype %}colSM{% endblock %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'django-admindocs-docroot' %}">{% trans 'Documentation' %}</a>
&rsaquo; {% trans 'Models' %}
</div>
{% endblock %}

{% block title %}Models{% endblock %}

{% block content %}

<h1>Model documentation</h1>

{% regroup models by app_label as grouped_models %}

<div id="content-main">
{% for group in grouped_models %}
<div class="module">
<h2 id="app-{{ group.grouper }}">{{ group.grouper|capfirst }}</h2>

<table class="xfull">
{% for model in group.list %}
<tr>
<th><a href="{% url 'django-admindocs-models-detail' app_label=model.app_label model_name=model.object_name.lower %}">{{ model.object_name }}</a></th>
</tr>
{% endfor %}
</table>
</div>
{% endfor %}

</div>
{% endblock %}

{% block sidebar %}
<div id="content-related" class="sidebar">
<div class="module">
<h2>Model groups</h2>
<ul>
{% regroup models by app_label as grouped_models %}
{% for group in grouped_models %}
    <li><a href="#app-{{ group.grouper }}">{{ group.grouper|capfirst }}</a></li>
{% endfor %}
</ul>
</div>
</div>
{% endblock %}
