{% extends "databrowse/base_site.html" %}

{% block title %}Databrowse{% endblock %}

{% block bodyid %}homepage{% endblock %}

{% block content %}

{% for model in model_list %}
  <div class="modelgroup {% cycle 'even' 'odd' %}">
	  <h2><a href="{{ model.url }}">{{ model.verbose_name_plural|capfirst }}</a></h2>
		<p>
		{% for object in model.sample_objects %}
			<a href="{{ object.url }}">{{ object }}</a>, 
		{% endfor %}
			<a class="more" href="{{ model.url }}">More &rarr;</a>
		</p>
  </div>
{% endfor %}

{% endblock %}
