# -*- coding: utf-8 -*-

"""
An alphabetical list of provinces for use as `choices` in a formfield.

Reference:
http://en.wikipedia.org/wiki/ISO_3166-2:CN
http://en.wikipedia.org/wiki/Province_%28China%29
http://en.wikipedia.org/wiki/Direct-controlled_municipality
http://en.wikipedia.org/wiki/Autonomous_regions_of_China
"""


CN_PROVINCE_CHOICES = (
    ("anhui", u"安徽"),
    ("beijing", u"北京"),
    ("chongqing", u"重庆"),
    ("fujian", u"福建"),
    ("gansu", u"甘肃"),
    ("guangdong", u"广东"),
    ("guangxi", u"广西壮族自治区"),
    ("guizhou", u"贵州"),
    ("hainan", u"海南"),
    ("hebei", u"河北"),
    ("heilongjiang", u"黑龙江"),
    ("henan", u"河南"),
    ("hongkong", u"香港"),
    ("hubei", u"湖北"),
    ("hunan", u"湖南"),
    ("jiangsu", u"江苏"),
    ("jiangxi", u"江西"),
    ("jilin", u"吉林"),
    ("liaoning", u"辽宁"),
    ("macao", u"澳门"),
    ("neimongol", u"内蒙古自治区"),
    ("ningxia", u"宁夏回族自治区"),
    ("qinghai", u"青海"),
    ("shaanxi", u"陕西"),
    ("shandong", u"山东"),
    ("shanghai", u"上海"),
    ("shanxi", u"山西"),
    ("sichuan", u"四川"),
    ("taiwan", u"台湾"),
    ("tianjin", u"天津"),
    ("xinjiang", u"新疆维吾尔自治区"),
    ("xizang", u"西藏自治区"),
    ("yunnan", u"云南"),
    ("zhejiang", u"浙江"),
)
