# -*- coding: utf-8 -*-
"""
A list of Colombian departaments as `choices` in a
formfield.

This exists in this standalone file so that it's only
imported into memory when explicitly needed.
"""

DEPARTMENT_CHOICES = (
    ('AMA', u'Amazonas'),
    ('ANT', u'Antioquia'),
    ('ARA', u'Arauca'),
    ('ATL', u'Atlántico'),
    ('DC',  u'Bogotá'),
    ('BOL', u'Bolívar'),
    ('BOY', u'Boyacá'),
    ('CAL', u'Caldas'),
    ('CAQ', u'Caquetá'),
    ('CAS', u'Casanare'),
    ('CAU', u'Cauca'),
    ('CES', u'Cesar'),
    ('CHO', u'Chocó'),
    ('COR', u'Córdoba'),
    ('CUN', u'Cundinamarca'),
    ('GUA', u'Guainía'),
    ('GUV', u'Guaviare'),
    ('HUI', u'Huila'),
    ('LAG', u'La Guajira'),
    ('MAG', u'Magdalena'),
    ('MET', u'Meta'),
    ('NAR', u'Nariño'),
    ('NSA', u'Norte de Santander'),
    ('PUT', u'Putumayo'),
    ('QUI', u'Quindío'),
    ('RIS', u'Risaralda'),
    ('SAP', u'San Andrés and Providencia'),
    ('SAN', u'Santander'),
    ('SUC', u'Sucre'),
    ('TOL', u'Tolima'),
    ('VAC', u'Valle del Cauca'),
    ('VAU', u'Vaupés'),
    ('VID', u'Vichada'),
)
