# -*- coding: utf-8 -*-
"""
Macedonian municipalities per the reorganization from 2004.
"""
from django.utils.translation import ugettext_lazy as _

MK_MUNICIPALITIES = (
    ('AD', _(u'Aerodrom')),
    ('AR', _(u'Aračinovo')),
    ('BR', _(u'Berovo')),
    ('TL', _(u'Bitola')),
    ('BG', _(u'Bogdanci')),
    ('VJ', _(u'Bogovinje')),
    ('BS', _(u'Bosilovo')),
    ('BN', _(u'Brvenica')),
    ('BU', _(u'Butel')),
    ('VA', _(u'Valandovo')),
    ('VL', _(u'Vasilevo')),
    ('VV', _(u'Vevčani')),
    ('VE', _(u'Veles')),
    ('NI', _(u'Vinica')),
    ('VC', _(u'Vraneštica')),
    ('VH', _(u'Vrapčište')),
    ('GB', _(u'Gazi Baba')),
    ('GV', _(u'Gevgelija')),
    ('GT', _(u'Gostivar')),
    ('GR', _(u'Gradsko')),
    ('DB', _(u'Debar')),
    ('DA', _(u'Debarca')),
    ('DL', _(u'Delčevo')),
    ('DK', _(u'Demir Kapija')),
    ('DM', _(u'Demir Hisar')),
    ('DE', _(u'Dolneni')),
    ('DR', _(u'Drugovo')),
    ('GP', _(u'Gjorče Petrov')),
    ('ZE', _(u'Želino')),
    ('ZA', _(u'Zajas')),
    ('ZK', _(u'Zelenikovo')),
    ('ZR', _(u'Zrnovci')),
    ('IL', _(u'Ilinden')),
    ('JG', _(u'Jegunovce')),
    ('AV', _(u'Kavadarci')),
    ('KB', _(u'Karbinci')),
    ('KX', _(u'Karpoš')),
    ('VD', _(u'Kisela Voda')),
    ('KH', _(u'Kičevo')),
    ('KN', _(u'Konče')),
    ('OC', _(u'Koćani')),
    ('KY', _(u'Kratovo')),
    ('KZ', _(u'Kriva Palanka')),
    ('KG', _(u'Krivogaštani')),
    ('KS', _(u'Kruševo')),
    ('UM', _(u'Kumanovo')),
    ('LI', _(u'Lipkovo')),
    ('LO', _(u'Lozovo')),
    ('MR', _(u'Mavrovo i Rostuša')),
    ('MK', _(u'Makedonska Kamenica')),
    ('MD', _(u'Makedonski Brod')),
    ('MG', _(u'Mogila')),
    ('NG', _(u'Negotino')),
    ('NV', _(u'Novaci')),
    ('NS', _(u'Novo Selo')),
    ('OS', _(u'Oslomej')),
    ('OD', _(u'Ohrid')),
    ('PE', _(u'Petrovec')),
    ('PH', _(u'Pehčevo')),
    ('PN', _(u'Plasnica')),
    ('PP', _(u'Prilep')),
    ('PT', _(u'Probištip')),
    ('RV', _(u'Radoviš')),
    ('RN', _(u'Rankovce')),
    ('RE', _(u'Resen')),
    ('RO', _(u'Rosoman')),
    ('AJ', _(u'Saraj')),
    ('SL', _(u'Sveti Nikole')),
    ('SS', _(u'Sopište')),
    ('SD', _(u'Star Dojran')),
    ('NA', _(u'Staro Nagoričane')),
    ('UG', _(u'Struga')),
    ('RU', _(u'Strumica')),
    ('SU', _(u'Studeničani')),
    ('TR', _(u'Tearce')),
    ('ET', _(u'Tetovo')),
    ('CE', _(u'Centar')),
    ('CZ', _(u'Centar-Župa')),
    ('CI', _(u'Čair')),
    ('CA', _(u'Čaška')),
    ('CH', _(u'Češinovo-Obleševo')),
    ('CS', _(u'Čučer-Sandevo')),
    ('ST', _(u'Štip')),
    ('SO', _(u'Šuto Orizari')),
)
