{% extends "admin/base_site.html" %}
{% load i18n %}
{% load url from future %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; {% trans 'Password reset' %}
</div>
{% endblock %}

{% block title %}{% trans 'Password reset successful' %}{% endblock %}

{% block content %}

<h1>{% trans 'Password reset successful' %}</h1>

<p>{% trans "We've e-mailed you instructions for setting your password to the e-mail address you submitted. You should be receiving it shortly." %}</p>

{% endblock %}
