# -*- coding: utf-8 -*-
"""A list of Urguayan departaments as `choices` in a formfield."""

DEPARTAMENT_CHOICES = (
    ('G', u'Artigas'),
    ('A', u'Canelones'),
    ('E', u'Cerro Largo'),
    ('L', u'Colonia'),
    ('Q', u'Durazno'),
    ('N', u'Flores'),
    ('O', u'Florida'),
    ('P', u'Lavalleja'),
    ('B', u'Maldonado'),
    ('S', u'Montevideo'),
    ('I', u'Paysandú'),
    ('J', u'Río Negro'),
    ('F', u'Rivera'),
    ('C', u'Rocha'),
    ('H', u'Salto'),
    ('M', u'San José'),
    ('K', u'Soriano'),
    ('R', u'Tacuarembó'),
    ('D', u'Treinta y Tres'),
)
