# -*- coding: utf-8 -*-
"""
An alphabetical list of Finnish municipalities for use as `choices` in a
formfield.

This exists in this standalone file so that it's only imported into memory
when explicitly needed.
"""

MUNICIPALITY_CHOICES = (
    ('akaa', u"Akaa"),
    ('alajarvi', u"Alajärvi"),
    ('alavieska', u"Alavieska"),
    ('alavus', u"Alavus"),
    ('artjarvi', u"Artjärvi"),
    ('asikkala', u"Asikkala"),
    ('askola', u"Askola"),
    ('aura', u"Aura"),
    ('brando', u"Brändö"),
    ('eckero', u"Eckerö"),
    ('enonkoski', u"Enonkoski"),
    ('enontekio', u"Enontekiö"),
    ('espoo', u"Espoo"),
    ('eura', u"Eura"),
    ('eurajoki', u"Eurajoki"),
    ('evijarvi', u"Evijärvi"),
    ('finstrom', u"Finström"),
    ('forssa', u"Forssa"),
    ('foglo', u"Föglö"),
    ('geta', u"Geta"),
    ('haapajarvi', u"Haapajärvi"),
    ('haapavesi', u"Haapavesi"),
    ('hailuoto', u"Hailuoto"),
    ('halsua', u"Halsua"),
    ('hamina', u"Hamina"),
    ('hammarland', u"Hammarland"),
    ('hankasalmi', u"Hankasalmi"),
    ('hanko', u"Hanko"),
    ('harjavalta', u"Harjavalta"),
    ('hartola', u"Hartola"),
    ('hattula', u"Hattula"),
    ('haukipudas', u"Haukipudas"),
    ('hausjarvi', u"Hausjärvi"),
    ('heinola', u"Heinola"),
    ('heinavesi', u"Heinävesi"),
    ('helsinki', u"Helsinki"),
    ('hirvensalmi', u"Hirvensalmi"),
    ('hollola', u"Hollola"),
    ('honkajoki', u"Honkajoki"),
    ('huittinen', u"Huittinen"),
    ('humppila', u"Humppila"),
    ('hyrynsalmi', u"Hyrynsalmi"),
    ('hyvinkaa', u"Hyvinkää"),
    ('hameenkoski', u"Hämeenkoski"),
    ('hameenkyro', u"Hämeenkyrö"),
    ('hameenlinna', u"Hämeenlinna"),
    ('ii', u"Ii"),
    ('iisalmi', u"Iisalmi"),
    ('iitti', u"Iitti"),
    ('ikaalinen', u"Ikaalinen"),
    ('ilmajoki', u"Ilmajoki"),
    ('ilomantsi', u"Ilomantsi"),
    ('imatra', u"Imatra"),
    ('inari', u"Inari"),
    ('inkoo', u"Inkoo"),
    ('isojoki', u"Isojoki"),
    ('isokyro', u"Isokyrö"),
    ('jalasjarvi', u"Jalasjärvi"),
    ('janakkala', u"Janakkala"),
    ('joensuu', u"Joensuu"),
    ('jokioinen', u"Jokioinen"),
    ('jomala', u"Jomala"),
    ('joroinen', u"Joroinen"),
    ('joutsa', u"Joutsa"),
    ('juankoski', u"Juankoski"),
    ('juuka', u"Juuka"),
    ('juupajoki', u"Juupajoki"),
    ('juva', u"Juva"),
    ('jyvaskyla', u"Jyväskylä"),
    ('jamijarvi', u"Jämijärvi"),
    ('jamsa', u"Jämsä"),
    ('jarvenpaa', u"Järvenpää"),
    ('kaarina', u"Kaarina"),
    ('kaavi', u"Kaavi"),
    ('kajaani', u"Kajaani"),
    ('kalajoki', u"Kalajoki"),
    ('kangasala', u"Kangasala"),
    ('kangasniemi', u"Kangasniemi"),
    ('kankaanpaa', u"Kankaanpää"),
    ('kannonkoski', u"Kannonkoski"),
    ('kannus', u"Kannus"),
    ('karijoki', u"Karijoki"),
    ('karjalohja', u"Karjalohja"),
    ('karkkila', u"Karkkila"),
    ('karstula', u"Karstula"),
    ('karttula', u"Karttula"),
    ('karvia', u"Karvia"),
    ('kaskinen', u"Kaskinen"),
    ('kauhajoki', u"Kauhajoki"),
    ('kauhava', u"Kauhava"),
    ('kauniainen', u"Kauniainen"),
    ('kaustinen', u"Kaustinen"),
    ('keitele', u"Keitele"),
    ('kemi', u"Kemi"),
    ('kemijarvi', u"Kemijärvi"),
    ('keminmaa', u"Keminmaa"),
    ('kemionsaari', u"Kemiönsaari"),
    ('kempele', u"Kempele"),
    ('kerava', u"Kerava"),
    ('kerimaki', u"Kerimäki"),
    ('kesalahti', u"Kesälahti"),
    ('keuruu', u"Keuruu"),
    ('kihnio', u"Kihniö"),
    ('kiikoinen', u"Kiikoinen"),
    ('kiiminki', u"Kiiminki"),
    ('kinnula', u"Kinnula"),
    ('kirkkonummi', u"Kirkkonummi"),
    ('kitee', u"Kitee"),
    ('kittila', u"Kittilä"),
    ('kiuruvesi', u"Kiuruvesi"),
    ('kivijarvi', u"Kivijärvi"),
    ('kokemaki', u"Kokemäki"),
    ('kokkola', u"Kokkola"),
    ('kolari', u"Kolari"),
    ('konnevesi', u"Konnevesi"),
    ('kontiolahti', u"Kontiolahti"),
    ('korsnas', u"Korsnäs"),
    ('koskitl', u"Koski Tl"),
    ('kotka', u"Kotka"),
    ('kouvola', u"Kouvola"),
    ('kristiinankaupunki', u"Kristiinankaupunki"),
    ('kruunupyy', u"Kruunupyy"),
    ('kuhmalahti', u"Kuhmalahti"),
    ('kuhmo', u"Kuhmo"),
    ('kuhmoinen', u"Kuhmoinen"),
    ('kumlinge', u"Kumlinge"),
    ('kuopio', u"Kuopio"),
    ('kuortane', u"Kuortane"),
    ('kurikka', u"Kurikka"),
    ('kustavi', u"Kustavi"),
    ('kuusamo', u"Kuusamo"),
    ('kylmakoski', u"Kylmäkoski"),
    ('kyyjarvi', u"Kyyjärvi"),
    ('karkola', u"Kärkölä"),
    ('karsamaki', u"Kärsämäki"),
    ('kokar', u"Kökar"),
    ('koylio', u"Köyliö"),
    ('lahti', u"Lahti"),
    ('laihia', u"Laihia"),
    ('laitila', u"Laitila"),
    ('lapinjarvi', u"Lapinjärvi"),
    ('lapinlahti', u"Lapinlahti"),
    ('lappajarvi', u"Lappajärvi"),
    ('lappeenranta', u"Lappeenranta"),
    ('lapua', u"Lapua"),
    ('laukaa', u"Laukaa"),
    ('lavia', u"Lavia"),
    ('lemi', u"Lemi"),
    ('lemland', u"Lemland"),
    ('lempaala', u"Lempäälä"),
    ('leppavirta', u"Leppävirta"),
    ('lestijarvi', u"Lestijärvi"),
    ('lieksa', u"Lieksa"),
    ('lieto', u"Lieto"),
    ('liminka', u"Liminka"),
    ('liperi', u"Liperi"),
    ('lohja', u"Lohja"),
    ('loimaa', u"Loimaa"),
    ('loppi', u"Loppi"),
    ('loviisa', u"Loviisa"),
    ('luhanka', u"Luhanka"),
    ('lumijoki', u"Lumijoki"),
    ('lumparland', u"Lumparland"),
    ('luoto', u"Luoto"),
    ('luumaki', u"Luumäki"),
    ('luvia', u"Luvia"),
    ('lansi-turunmaa', u"Länsi-Turunmaa"),
    ('maalahti', u"Maalahti"),
    ('maaninka', u"Maaninka"),
    ('maarianhamina', u"Maarianhamina"),
    ('marttila', u"Marttila"),
    ('masku', u"Masku"),
    ('merijarvi', u"Merijärvi"),
    ('merikarvia', u"Merikarvia"),
    ('miehikkala', u"Miehikkälä"),
    ('mikkeli', u"Mikkeli"),
    ('muhos', u"Muhos"),
    ('multia', u"Multia"),
    ('muonio', u"Muonio"),
    ('mustasaari', u"Mustasaari"),
    ('muurame', u"Muurame"),
    ('mynamaki', u"Mynämäki"),
    ('myrskyla', u"Myrskylä"),
    ('mantsala', u"Mäntsälä"),
    ('mantta-vilppula', u"Mänttä-Vilppula"),
    ('mantyharju', u"Mäntyharju"),
    ('naantali', u"Naantali"),
    ('nakkila', u"Nakkila"),
    ('nastola', u"Nastola"),
    ('nilsia', u"Nilsiä"),
    ('nivala', u"Nivala"),
    ('nokia', u"Nokia"),
    ('nousiainen', u"Nousiainen"),
    ('nummi-pusula', u"Nummi-Pusula"),
    ('nurmes', u"Nurmes"),
    ('nurmijarvi', u"Nurmijärvi"),
    ('narpio', u"Närpiö"),
    ('oravainen', u"Oravainen"),
    ('orimattila', u"Orimattila"),
    ('oripaa', u"Oripää"),
    ('orivesi', u"Orivesi"),
    ('oulainen', u"Oulainen"),
    ('oulu', u"Oulu"),
    ('oulunsalo', u"Oulunsalo"),
    ('outokumpu', u"Outokumpu"),
    ('padasjoki', u"Padasjoki"),
    ('paimio', u"Paimio"),
    ('paltamo', u"Paltamo"),
    ('parikkala', u"Parikkala"),
    ('parkano', u"Parkano"),
    ('pedersore', u"Pedersöre"),
    ('pelkosenniemi', u"Pelkosenniemi"),
    ('pello', u"Pello"),
    ('perho', u"Perho"),
    ('pertunmaa', u"Pertunmaa"),
    ('petajavesi', u"Petäjävesi"),
    ('pieksamaki', u"Pieksämäki"),
    ('pielavesi', u"Pielavesi"),
    ('pietarsaari', u"Pietarsaari"),
    ('pihtipudas', u"Pihtipudas"),
    ('pirkkala', u"Pirkkala"),
    ('polvijarvi', u"Polvijärvi"),
    ('pomarkku', u"Pomarkku"),
    ('pori', u"Pori"),
    ('pornainen', u"Pornainen"),
    ('porvoo', u"Porvoo"),
    ('posio', u"Posio"),
    ('pudasjarvi', u"Pudasjärvi"),
    ('pukkila', u"Pukkila"),
    ('punkaharju', u"Punkaharju"),
    ('punkalaidun', u"Punkalaidun"),
    ('puolanka', u"Puolanka"),
    ('puumala', u"Puumala"),
    ('pyhtaa', u"Pyhtää"),
    ('pyhajoki', u"Pyhäjoki"),
    ('pyhajarvi', u"Pyhäjärvi"),
    ('pyhanta', u"Pyhäntä"),
    ('pyharanta', u"Pyhäranta"),
    ('palkane', u"Pälkäne"),
    ('poytya', u"Pöytyä"),
    ('raahe', u"Raahe"),
    ('raasepori', u"Raasepori"),
    ('raisio', u"Raisio"),
    ('rantasalmi', u"Rantasalmi"),
    ('ranua', u"Ranua"),
    ('rauma', u"Rauma"),
    ('rautalampi', u"Rautalampi"),
    ('rautavaara', u"Rautavaara"),
    ('rautjarvi', u"Rautjärvi"),
    ('reisjarvi', u"Reisjärvi"),
    ('riihimaki', u"Riihimäki"),
    ('ristiina', u"Ristiina"),
    ('ristijarvi', u"Ristijärvi"),
    ('rovaniemi', u"Rovaniemi"),
    ('ruokolahti', u"Ruokolahti"),
    ('ruovesi', u"Ruovesi"),
    ('rusko', u"Rusko"),
    ('raakkyla', u"Rääkkylä"),
    ('saarijarvi', u"Saarijärvi"),
    ('salla', u"Salla"),
    ('salo', u"Salo"),
    ('saltvik', u"Saltvik"),
    ('sastamala', u"Sastamala"),
    ('sauvo', u"Sauvo"),
    ('savitaipale', u"Savitaipale"),
    ('savonlinna', u"Savonlinna"),
    ('savukoski', u"Savukoski"),
    ('seinajoki', u"Seinäjoki"),
    ('sievi', u"Sievi"),
    ('siikainen', u"Siikainen"),
    ('siikajoki', u"Siikajoki"),
    ('siikalatva', u"Siikalatva"),
    ('siilinjarvi', u"Siilinjärvi"),
    ('simo', u"Simo"),
    ('sipoo', u"Sipoo"),
    ('siuntio', u"Siuntio"),
    ('sodankyla', u"Sodankylä"),
    ('soini', u"Soini"),
    ('somero', u"Somero"),
    ('sonkajarvi', u"Sonkajärvi"),
    ('sotkamo', u"Sotkamo"),
    ('sottunga', u"Sottunga"),
    ('sulkava', u"Sulkava"),
    ('sund', u"Sund"),
    ('suomenniemi', u"Suomenniemi"),
    ('suomussalmi', u"Suomussalmi"),
    ('suonenjoki', u"Suonenjoki"),
    ('sysma', u"Sysmä"),
    ('sakyla', u"Säkylä"),
    ('taipalsaari', u"Taipalsaari"),
    ('taivalkoski', u"Taivalkoski"),
    ('taivassalo', u"Taivassalo"),
    ('tammela', u"Tammela"),
    ('tampere', u"Tampere"),
    ('tarvasjoki', u"Tarvasjoki"),
    ('tervo', u"Tervo"),
    ('tervola', u"Tervola"),
    ('teuva', u"Teuva"),
    ('tohmajarvi', u"Tohmajärvi"),
    ('toholampi', u"Toholampi"),
    ('toivakka', u"Toivakka"),
    ('tornio', u"Tornio"),
    ('turku', u"Turku"),
    ('tuusniemi', u"Tuusniemi"),
    ('tuusula', u"Tuusula"),
    ('tyrnava', u"Tyrnävä"),
    ('toysa', u"Töysä"),
    ('ulvila', u"Ulvila"),
    ('urjala', u"Urjala"),
    ('utajarvi', u"Utajärvi"),
    ('utsjoki', u"Utsjoki"),
    ('uurainen', u"Uurainen"),
    ('uusikaarlepyy', u"Uusikaarlepyy"),
    ('uusikaupunki', u"Uusikaupunki"),
    ('vaala', u"Vaala"),
    ('vaasa', u"Vaasa"),
    ('valkeakoski', u"Valkeakoski"),
    ('valtimo', u"Valtimo"),
    ('vantaa', u"Vantaa"),
    ('varkaus', u"Varkaus"),
    ('varpaisjarvi', u"Varpaisjärvi"),
    ('vehmaa', u"Vehmaa"),
    ('vesanto', u"Vesanto"),
    ('vesilahti', u"Vesilahti"),
    ('veteli', u"Veteli"),
    ('vierema', u"Vieremä"),
    ('vihanti', u"Vihanti"),
    ('vihti', u"Vihti"),
    ('viitasaari', u"Viitasaari"),
    ('vimpeli', u"Vimpeli"),
    ('virolahti', u"Virolahti"),
    ('virrat', u"Virrat"),
    ('vardo', u"Vårdö"),
    ('vahakyro', u"Vähäkyrö"),
    ('voyri-maksamaa', u"Vöyri-Maksamaa"),
    ('yli-ii', u"Yli-Ii"),
    ('ylitornio', u"Ylitornio"),
    ('ylivieska', u"Ylivieska"),
    ('ylojarvi', u"Ylöjärvi"),
    ('ypaja', u"Ypäjä"),
    ('ahtari', u"Ähtäri"),
    ('aanekoski', u"Äänekoski")
)